"""MCP Ticketer - Universal ticket management interface."""

from .__version__ import (
                          __author__,
                          __author_email__,
                          __copyright__,
                          __description__,
                          __license__,
                          __title__,
                          __version__,
                          __version_info__,
                          get_user_agent,
                          get_version,
)

__all__ = [
    "__version__",
    "__version_info__",
    "__title__",
    "__description__",
    "__author__",
    "__author_email__",
    "__license__",
    "__copyright__",
    "get_version",
    "get_user_agent",
]
